@@ -1,6 +1,6 @@
 /*
  * Hurl (https://hurl.dev)
- * Copyright (C) 2024 Orange
+ * Copyright (C) 2025 Orange
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,6 +15,7 @@
  * limitations under the License.
  *
  */
+use super::placeholder;
 use crate::ast::{JsonListElement, JsonObjectElement, JsonValue, SourceInfo, Template};
 use crate::combinator::{choice, non_recover, ParseError as ParseErrorTrait};
 use crate::parser::primitives::{boolean, hex_digit, literal, try_literal};
@@ -22,8 +23,6 @@ use crate::parser::template::EncodedString;
 use crate::parser::{templatize, JsonErrorVariant, ParseError, ParseErrorKind, ParseResult};
 use crate::reader::{Pos, Reader};
 
-use super::placeholder;
-
 pub fn parse(reader: &mut Reader) -> ParseResult<JsonValue> {
     choice(
         &[
@@ -100,11 +99,7 @@ fn string_template(reader: &mut Reader) -> ParseResult<Template> {
     literal("\"", reader)?;
     let elements = templatize(encoded_string)?;
 
-    let template = Template {
-        delimiter,
-        elements,
-        source_info: SourceInfo::new(start.pos, end.pos),
-    };
+    let template = Template::new(delimiter, elements, SourceInfo::new(start.pos, end.pos));
     Ok(template)
 }
 
@@ -213,44 +208,53 @@ fn hex_value(reader: &mut Reader) -> ParseResult<u32> {
 }
 
 pub fn number_value(reader: &mut Reader) -> ParseResult<JsonValue> {
-    let start = reader.cursor();
-
     let sign = match try_literal("-", reader) {
         Err(_) => String::new(),
         Ok(_) => "-".to_string(),
     };
+    let int = integer(reader)?;
+    let frac = fraction(reader)?;
+    let exp = exponent(reader)?;
+    Ok(JsonValue::Number(format!("{sign}{int}{frac}{exp}")))
+}
 
-    let integer = match try_literal("0", reader) {
+fn integer(reader: &mut Reader) -> ParseResult<String> {
+    let start = reader.cursor();
+    match try_literal("0", reader) {
         Err(_) => {
             let digits = reader.read_while(|c| c.is_ascii_digit());
             if digits.is_empty() {
                 let kind = ParseErrorKind::Expecting {
                     value: "number".to_string(),
                 };
-                return Err(ParseError::new(start.pos, true, kind));
+                Err(ParseError::new(start.pos, true, kind))
             } else {
-                digits
+                Ok(digits)
             }
         }
-        Ok(_) => "0".to_string(),
-    };
+        Ok(_) => Ok("0".to_string()),
+    }
+}
 
-    let fraction = match try_literal(".", reader) {
+fn fraction(reader: &mut Reader) -> ParseResult<String> {
+    match try_literal(".", reader) {
         Ok(_) => {
             let digits = reader.read_while(|c| c.is_ascii_digit());
             if digits.is_empty() {
                 let kind = ParseErrorKind::Expecting {
                     value: "digits".to_string(),
                 };
-                return Err(ParseError::new(reader.cursor().pos, false, kind));
+                Err(ParseError::new(reader.cursor().pos, false, kind))
             } else {
-                format!(".{digits}")
+                Ok(format!(".{digits}"))
             }
         }
-        Err(_) => String::new(),
-    };
+        Err(_) => Ok(String::new()),
+    }
+}
 
-    let exponent = if reader.peek() == Some('e') || reader.peek() == Some('E') {
+fn exponent(reader: &mut Reader) -> ParseResult<String> {
+    if reader.peek() == Some('e') || reader.peek() == Some('E') {
         reader.read();
         let exponent_sign = match try_literal("-", reader) {
             Ok(_) => "-".to_string(),
@@ -260,14 +264,10 @@ pub fn number_value(reader: &mut Reader) -> ParseResult<JsonValue> {
             },
         };
         let exponent_digits = reader.read_while(|c| c.is_ascii_digit());
-        format!("e{exponent_sign}{exponent_digits}")
+        Ok(format!("e{exponent_sign}{exponent_digits}"))
     } else {
-        String::new()
-    };
-
-    Ok(JsonValue::Number(format!(
-        "{sign}{integer}{fraction}{exponent}"
-    )))
+        Ok(String::new())
+    }
 }
 
 fn expression_value(reader: &mut Reader) -> ParseResult<JsonValue> {
@@ -409,6 +409,7 @@ fn whitespace(reader: &mut Reader) -> String {
 mod tests {
     use super::*;
     use crate::ast::*;
+    use crate::typing::ToSource;
 
     #[test]
     fn test_parse_error() {
@@ -449,6 +450,35 @@ mod tests {
         assert!(error.recoverable);
     }
 
+    #[test]
+    fn test_integer() {
+        let mut reader = Reader::new("0");
+        assert_eq!(integer(&mut reader).unwrap(), "0".to_string());
+        assert_eq!(reader.cursor().index, 1);
+
+        let mut reader = Reader::new("123");
+        assert_eq!(integer(&mut reader).unwrap(), "123".to_string());
+        assert_eq!(reader.cursor().index, 3);
+
+        let mut reader = Reader::new("0123");
+        assert_eq!(integer(&mut reader).unwrap(), "0".to_string());
+        assert_eq!(reader.cursor().index, 1);
+    }
+
+    #[test]
+    fn test_fraction() {
+        let mut reader = Reader::new(".5");
+        assert_eq!(fraction(&mut reader).unwrap(), ".5".to_string());
+        assert_eq!(reader.cursor().index, 2);
+    }
+
+    #[test]
+    fn test_exponent() {
+        let mut reader = Reader::new("e2");
+        assert_eq!(exponent(&mut reader).unwrap(), "e2".to_string());
+        assert_eq!(reader.cursor().index, 2);
+    }
+
     #[test]
     fn test_boolean_value() {
         let mut reader = Reader::new("true");
@@ -472,12 +502,12 @@ mod tests {
 
     pub fn json_hello_world_value() -> JsonValue {
         // "hello\u0020{{name}}!"
-        JsonValue::String(Template {
-            delimiter: Some('"'),
-            elements: vec![
+        JsonValue::String(Template::new(
+            Some('"'),
+            vec![
                 TemplateElement::String {
                     value: "Hello ".to_string(),
-                    encoded: "Hello\\u0020".to_string(),
+                    source: "Hello\\u0020".to_source(),
                 },
                 TemplateElement::Placeholder(Placeholder {
                     space0: Whitespace {
@@ -498,11 +528,11 @@ mod tests {
                 }),
                 TemplateElement::String {
                     value: "!".to_string(),
-                    encoded: "!".to_string(),
+                    source: "!".to_source(),
                 },
             ],
-            source_info: SourceInfo::new(Pos::new(1, 2), Pos::new(1, 22)),
-        })
+            SourceInfo::new(Pos::new(1, 2), Pos::new(1, 22)),
+        ))
     }
 
     #[test]
@@ -510,11 +540,11 @@ mod tests {
         let mut reader = Reader::new("\"\"");
         assert_eq!(
             string_value(&mut reader).unwrap(),
-            JsonValue::String(Template {
-                delimiter: Some('"'),
-                elements: vec![],
-                source_info: SourceInfo::new(Pos::new(1, 2), Pos::new(1, 2)),
-            })
+            JsonValue::String(Template::new(
+                Some('"'),
+                vec![],
+                SourceInfo::new(Pos::new(1, 2), Pos::new(1, 2)),
+            ))
         );
         assert_eq!(reader.cursor().index, 2);
 
@@ -525,14 +555,14 @@ mod tests {
         let mut reader = Reader::new("\"{}\"");
         assert_eq!(
             string_value(&mut reader).unwrap(),
-            JsonValue::String(Template {
-                delimiter: Some('"'),
-                elements: vec![TemplateElement::String {
+            JsonValue::String(Template::new(
+                Some('"'),
+                vec![TemplateElement::String {
                     value: "{}".to_string(),
-                    encoded: "{}".to_string(),
+                    source: "{}".to_source(),
                 }],
-                source_info: SourceInfo::new(Pos::new(1, 2), Pos::new(1, 4)),
-            })
+                SourceInfo::new(Pos::new(1, 2), Pos::new(1, 4)),
+            ))
         );
         assert_eq!(reader.cursor().index, 4);
     }
@@ -907,14 +937,14 @@ mod tests {
                 space0: "\n  ".to_string(),
                 elements: vec![JsonObjectElement {
                     space0: String::new(),
-                    name: Template {
-                        delimiter: Some('"'),
-                        elements: vec![TemplateElement::String {
+                    name: Template::new(
+                        Some('"'),
+                        vec![TemplateElement::String {
                             value: "a".to_string(),
-                            encoded: "a".to_string()
+                            source: "a".to_source()
                         }],
-                        source_info: SourceInfo::new(Pos::new(2, 4), Pos::new(2, 5))
-                    },
+                        SourceInfo::new(Pos::new(2, 4), Pos::new(2, 5))
+                    ),
                     space1: String::new(),
                     space2: " ".to_string(),
                     value: JsonValue::Boolean(true),
@@ -955,14 +985,14 @@ mod tests {
             object_element(&mut reader).unwrap(),
             JsonObjectElement {
                 space0: String::new(),
-                name: Template {
-                    delimiter: Some('"'),
-                    elements: vec![TemplateElement::String {
+                name: Template::new(
+                    Some('"'),
+                    vec![TemplateElement::String {
                         value: "a".to_string(),
-                        encoded: "a".to_string()
+                        source: "a".to_source()
                     }],
-                    source_info: SourceInfo::new(Pos::new(1, 2), Pos::new(1, 3))
-                },
+                    SourceInfo::new(Pos::new(1, 2), Pos::new(1, 3))
+                ),
                 space1: String::new(),
                 space2: " ".to_string(),
                 value: JsonValue::Boolean(true),
